//----------------------------------------------------------------------------
// Borland BIDS Container Library Compatibility header
// Copyright (c) 1998-1999, OWL NExt
//
//$Revision: 3 $
// Version 1.5 - Timothy Byrd
//     Implemented barebones TArrayAsVector, TSArrayAsVector,
//     TIArrayAsVector and TISArrayAsVector along with iterators.
//     To implement other forms of Arrays, see the code below
//     and the code in deques.h and queues.h
//----------------------------------------------------------------------------
#if !defined( CLASSLIB_ARRAYS_H )
#define CLASSLIB_ARRAYS_H

#pragma message ("Included obsolete file: <classlib/arrays.h> use STL or <owl/template.h> instead")

#include <classlib/defs.h>

// if use <owl/template.h>
#if defined(BI_NO_EMULATESTL)

#   include <owl/template.h>
    // include compatibility classes
#   include <classlib/compat.h>

#else

#define CLASSLIB_COMPAT_H

#include "owl\shddel.h"
#include <vector>


__OWL_BEGIN_NAMESPACE

// Generic definitions/compiler options (eg. alignment) preceeding the
// definition of classes
#include <owl/preclass.h>

template<class T> class TArrayAsVector : public std::vector<T> {
public:
    typedef void (*IterFunc)(T&, void*);
//    typedef bool (*CondFunc)(const T&, void*);

    explicit TArrayAsVector(unsigned int init_size = 0)
        : d_upper(init_size), d_lower(0), d_delta(0)
    {
        if (init_size > 0) {
            reserve(init_size);
        }
    }

    TArrayAsVector(unsigned int init_size, int lower_bound, unsigned int delta = 0)
        : d_upper(init_size), d_lower(lower_bound), d_delta(delta)
    {
        if (init_size > 0) {
            reserve(init_size);
        }
    }

    T& operator []( int loc )
    {
        Grow( loc );
        return at(ZeroBase(loc));
    }

    const T& operator []( int loc ) const
    {
        PRECONDITION( loc >= d_lower && ZeroBase(loc) < size() );
        return at(ZeroBase(loc));
    }

    bool Add(const T& t) {
        if ( d_delta > 0 && size() >= capacity()) {
            reserve(size() + d_delta);
        }
        push_back(t);
        return true;
    }

    bool AddAt(const T& t, uint index) {
        int zindex = ZeroBase(index);
        if (zindex < 0 || zindex >= size()) {
            Grow(index);
            zindex = ZeroBase(index);
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
            PRECONDITION(zindex >= 0 && zindex <= size());
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
            at(zindex) = t;
        }
        else {
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
            PRECONDITION(zindex >= 0 && zindex <= size());
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
            insert(begin()+zindex, t);
        }
        return true;
    }

    unsigned int ArraySize() const
    {
        return capacity();
    }

    int BoundBase( int loc ) const
    {
        return loc == -1 || loc == UINT_MAX ? INT_MAX : loc + d_lower;
    }

    int Destroy( const T& t )
    {
        return Detach(t);
    }

    int Destroy( int loc )
    {
        return Detach(loc);
    }

    int Detach(const T& t)
    {
        return _Detach(_Find(t));
    }

    int Detach( int loc )
    {
        return _Detach(ZeroBase(loc));
    }

    int Find( const T& t ) const
    {
        return BoundBase(_Find(t));
    }

    void Flush()
    {
        erase(begin(),end());
    }

    void ForEach(IterFunc iter, void* args)
    {
        int last = size();
        for(int i = 0; i < last; ++i)
            (iter)(at(i), args);
    }

    unsigned int GetItemsInContainer() const
    {
        return size();
    }

    void Grow( int loc )
    {
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
        int new_size = ZeroBase(loc);
        if ( new_size >= (int) size() ) {
            if (d_delta) {
                reserve(new_size + d_delta - (new_size % d_delta));
            }
            ++new_size;
            resize(new_size);
        }
        else if (new_size < 0) {
            insert(begin(), -new_size, T());
            d_lower += new_size;
        }
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
    }

    bool HasMember(const T& t) const
    {
        return _Find(t) != -1;
    }

    bool IsEmpty() const
    {
        return empty();
    }

    bool IsFull() const
    {
        return d_delta == 0 && size() > ZeroBase(d_upper);
    }

    int LowerBound() const
    {
        return d_lower;
    }

    bool Reallocate( unsigned sz, unsigned offset = 0 )
    {
        if (!d_delta || sz <= size()) {
            return false;
        }
        --sz;
        sz = sz + d_delta - (sz % d_delta);
        if (sz > capacity()) {
            reserve(sz);
        }
        if (offset > 0) {
            insert(begin(), offset, T());
        }
        if (size() < sz) {
            resize(sz);
        }
        return true;
    }

    void RemoveEntry( int loc )
    {
        SqueezeEntry( (uint)loc );
    }

    void SetData( int loc, const T& t )
    {
        loc = ZeroBase(loc);
        PRECONDITION(loc >= 0 && loc <= size());
        at(loc) = t;
    }

    void SqueezeEntry( unsigned loc )
    {
        PRECONDITION( loc <= size() );
        Detach( (int)loc );
    }

    int UpperBound() const
    {
        return d_lower + size() - 1;
    }

    int ZeroBase( int loc ) const
    {
        return loc - d_lower;
    }

protected:
    int _Find(const T& t) const
    {
        int i = size() - 1;
        while (i >= 0) {
            if (at(i) == t) break;
            --i;
        }
        return i;
    }

    int _Detach(int i)
    {
        if (i < 0 || ((unsigned) i) >= size())
            return 0;
        erase(begin()+i);
        return 1;
    }
    
    int d_lower;
    int d_upper;
    unsigned int d_delta;
};


template<class T> class TArrayAsVectorIterator {
public:

    typedef TArrayAsVector<T> abase;
    typedef TArrayAsVectorIterator<T> base;
    
    TArrayAsVectorIterator(const abase& a)
        : d_a(&a), d_cur(0), d_lower(0), d_upper(a.size())
    { };

    TArrayAsVectorIterator(const abase& a, unsigned start, unsigned stop)
        : d_a(&a), d_cur(start), d_lower(start), d_upper(stop)
    {
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
        PRECONDITION(0 <= d_cur);
        PRECONDITION(d_cur <= d_upper);
        PRECONDITION(d_upper <= d_a->size());
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
    };

    operator bool() const { return notdone(); }

    const T& operator ++(int) {
        d_cur++;
        PRECONDITION(d_cur <= d_upper);
        return d_a->at(d_cur-1);
    }

    const T& operator ++() {
        ++d_cur;
        PRECONDITION(d_cur < d_upper);
        return d_a->at(d_cur);
    }

    const T& Current() const {
        PRECONDITION(d_cur < d_upper);
        return d_a->at(d_cur);
    }

    void Restart() { Restart(d_lower, d_upper); };

    void Restart(unsigned start, unsigned stop) {
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
        PRECONDITION(0 <= start);
        PRECONDITION(start <= stop);
        PRECONDITION(stop <= d_a->size());
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
        d_cur = d_lower = start;
        d_upper = stop;
    }

protected:
    bool notdone() const { return d_cur < d_upper /* && d_cur < Vect.Size() */; }
    const abase* d_a;
    unsigned d_cur;
    unsigned d_upper;
    unsigned d_lower;
};


template<class T> class TSArrayAsVector : public TArrayAsVector<T> {
public:
    explicit TSArrayAsVector(unsigned int init_size = 0)
        : TArrayAsVector<T>(init_size)
    { };

    TSArrayAsVector(unsigned int init_size, int lower_bound, unsigned int delta = 0)
        : TArrayAsVector<T>(init_size, lower_bound, delta)
    { };

    bool Add(const T& t) {
        int i = size() - 1;
        push_back(t);
        while (0 <= i && t < at(i)) {
            at(i+1) = at(i);
            --i;
        }
        if ((unsigned)(i + 2) < size()) {
            at(i+1) = t;
        }
        return true;
    }

private:
    bool AddAt(const T&) { return false; };
};


template<class T> class TSArrayAsVectorIterator : public TArrayAsVectorIterator<T> {
public:
    TSArrayAsVectorIterator(const TSArrayAsVector<T>& a)
        : TArrayAsVectorIterator<T>(a)
    { };

    TSArrayAsVectorIterator(const TSArrayAsVector<T>& a, unsigned start, unsigned stop)
        : TArrayAsVectorIterator<T>(a, start, stop)
    { };
};


template<class T> class TIArrayAsVector : public TArrayAsVector<T*>, public TShouldDelete {
public:
    typedef void (*IterFunc)(T&, void*);
//    typedef bool (*CondFunc)(const T&, void*);

    explicit TIArrayAsVector(unsigned int init_size = 0)
        : TArrayAsVector<T*>(init_size)
    { };

    TIArrayAsVector(unsigned int init_size, int lower_bound, unsigned int delta = 0)
        : TArrayAsVector<T*>(init_size, lower_bound, delta)
    { };

    ~TIArrayAsVector() {
        Flush();
    }

    T* &operator []( int loc )
    {
        Grow( loc );
        return at(ZeroBase(loc));
    }

    T* const &operator []( int loc ) const
    {
        PRECONDITION( loc >= d_lower && ZeroBase(loc) < size() );
        return at(ZeroBase(loc));
    }

    bool AddAt(T* const t, uint index) {
        int zindex = ZeroBase(index);
        if (zindex < 0 || zindex >= size()) {
            Grow(index);
            zindex = ZeroBase(index);
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
            PRECONDITION(zindex >= 0 && zindex <= size());
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
            at(zindex) = t;
        }
        else {
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
            PRECONDITION(zindex >= 0 && zindex <= size());
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
            insert(begin()+zindex, t);
        }
        return true;
    }

    int Destroy(const T *t)
    {
        return _Detach(_Find(t), TShouldDelete::Delete);
    }

    int Destroy( int loc )
    {
        return _Detach(ZeroBase(loc), TShouldDelete::Delete);
    }

    int Detach(const T* t, DeleteType dt = NoDelete)
    {
        return _Detach(_Find(t), dt);
    }

    int Detach(int loc, DeleteType dt = NoDelete)
    {
        return _Detach(ZeroBase(loc), dt);
    }

    int Find(const T* t) const
    {
        return BoundBase(_Find(t));
    }

    void Flush( DeleteType dt = DefDelete )
    {
        if (DelObj(dt)) {
            for (int i = size() - 1; i >= 0; --i) {
                delete at(i);
                at(i) = 0;
            }
        }
        erase(begin(),end());
    }

    void ForEach(IterFunc iter, void* args)
    {
        int last = size();
        for(int i = 0; i < last; ++i)
            (iter)(*at(i), args);
    }

    void Grow( int loc )
    {
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
        int new_size = ZeroBase(loc);
        if ( new_size >= (int) size() ) {
            if (d_delta) {
                reserve(new_size + d_delta - (new_size % d_delta));
            }
            ++new_size;
            resize(new_size,0);
        }
        else if (new_size < 0) {
            insert(begin(), -new_size, 0);
            d_lower += new_size;
        }
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
    }

    bool HasMember(const T* t) const
    {
        return _Find(t) != -1;
    }

    bool Reallocate( unsigned sz, unsigned offset = 0 )
    {
        if (!d_delta || sz <= size()) {
            return false;
        }
        --sz;
        sz = sz + d_delta - (sz % d_delta);
        if (sz > capacity()) {
            reserve(sz);
        }
        if (offset > 0) {
            insert(begin(), offset, 0);
        }
        if (size() < sz) {
            resize(sz, 0);
        }
        return true;
    }

protected:
    int _Find(const T* t) const
    {
        int i = size() - 1;
        while (i >= 0) {
            if (at(i) == t) break;
            --i;
        }
        return i;
    }

    int _Detach(int i, DeleteType dt)
    {
        if (i < 0 || ((unsigned) i) >= size())
            return 0;
        if (DelObj(dt)) {
            T* t = at(i);
            delete t;
        }
        erase(begin()+i);
        return 1;
    }
};


template<class T> class TIArrayAsVectorIterator {
public:

    typedef TIArrayAsVector<T> abase;
    typedef TIArrayAsVectorIterator<T> base;

    TIArrayAsVectorIterator(const abase& a)
        : d_a(&a), d_cur(0), d_lower(0), d_upper(a.size())
    { };

    TIArrayAsVectorIterator(const abase& a, unsigned start, unsigned stop)
        : d_a(&a), d_cur(start), d_lower(start), d_upper(stop)
    {
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
        PRECONDITION(0 <= d_cur);
        PRECONDITION(d_cur <= d_upper);
        PRECONDITION(d_upper <= d_a->size());
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
    };

    operator bool() const { return notdone(); }

    T* Current() const {
        PRECONDITION(d_cur < d_upper);
        return const_cast<T*>(d_a->at(d_cur));
    }

    T* operator ++(int) {
        d_cur++;
        PRECONDITION(d_cur <= d_upper);
        return const_cast<T*>(d_a->at(d_cur-1));
    }

    T* operator ++() {
        ++d_cur;
        PRECONDITION(d_cur < d_upper);
        return const_cast<T*>(d_a->at(d_cur));
    }

    void Restart() { Restart(d_lower, d_upper); };

    void Restart(unsigned start, unsigned stop) {
#if defined(__BORLANDC__)
#pragma warn -ccc
#endif
        PRECONDITION(0 <= start);
        PRECONDITION(start <= stop);
        PRECONDITION(stop <= d_a->size());
#if defined(__BORLANDC__)
#pragma warn .ccc
#endif
        d_cur = d_lower = start;
        d_upper = stop;
    }

protected:
    bool notdone() const { return d_cur < d_upper /* && d_cur < Vect.Size() */; }
    const abase* d_a;
    unsigned d_cur;
    unsigned d_upper;
    unsigned d_lower;
};


template<class T> class TISArrayAsVector : public TIArrayAsVector<T> {
public:
    explicit TISArrayAsVector(unsigned int init_size = 0)
        : TIArrayAsVector<T>(init_size)
    { };

    TISArrayAsVector(unsigned int init_size, int lower_bound, unsigned int delta = 0)
        : TIArrayAsVector<T>(init_size, lower_bound, delta)
    { };

    bool Add(T* t) {
        int i = size() - 1;
        push_back(t);
        while (0 <= i && t && (!at(i) || *t < *at(i))) {
            at(i+1) = at(i);
            --i;
        }
        if ((unsigned)(i + 2) < size()) {
            at(i+1) = t;
        }
        return true;
    }

private:
    bool AddAt(const T&) { return false; };
};


template<class T> class TISArrayAsVectorIterator : public TIArrayAsVectorIterator<T> {
public:
    TISArrayAsVectorIterator(const TISArrayAsVector<T>& a)
        : TIArrayAsVectorIterator<T>(a)
        { };

    TISArrayAsVectorIterator(const TISArrayAsVector<T>& a, unsigned start, unsigned stop)
        : TIArrayAsVectorIterator<T>(a, start, stop)
        { };
};

// Generic definitions/compiler options (eg. alignment) following the
// definition of classes
#include <owl/posclass.h>

#define TArray                  TArrayAsVector
#define TArrayIterator          TArrayAsVectorIterator
#define TSArray                 TSArrayAsVector
#define TSArrayIterator         TSArrayAsVectorIterator
#define TIArray                 TIArrayAsVector
#define TIArrayIterator         TIArrayAsVectorIterator
#define TISArray                TISArrayAsVector
#define TISArrayIterator        TISArrayAsVectorIterator

__OWL_END_NAMESPACE

#endif  // BI_USE_TEMPLATE


#endif  // CLASSLIB_ARRAYS_H
